/********************************************************************************
*Module name:
* I2C_ROUTINES.
*
*Module version:
* 1.00. Initial version.
*
*Module description:
* Includes the necessary functions the I2C protocol needs for its correct 
* functionality
********************************************************************************/



/*Header files.*/
/********************************************************************************/
/********************************************************************************/
#include "project.h"


/*Local functions at module*/
/********************************************************************************/
/********************************************************************************/
void I2C__P_StartCondition(void);
void I2C__P_ReStartCondition(void);
uint8_t I2C__P_SendByte(uint8_t Byte);
uint8_t I2C__P_GetACK(void);
void I2C__P_StopCondition(void);
void I2C__P_Clock(void);
uint8_t I2C__P_GetByte(void);
void I2C__P_SendACK(void);
void I2C__P_NACK(void);



#define DELAY_SHORT 10
#define DELAY_LONG 20

void swi2c_init(void)
{
	const cy_stc_gpio_prt_config_t port6_cfg =
	{
		.out        = 0x00000008u,
		.intrMask   = 0x00000030u,
		.intrCfg    = 0x00000D00u,
		.cfg        = 0x00886666u,
		.cfgIn      = 0x00000000u,
		.cfgOut     = 0x00000000u,
		.cfgSIO     = 0x00000000u,
		.sel0Active = 0x00000000u,
		.sel1Active = 0x00000000u,
	};
	(void)Cy_GPIO_Port_Init(GPIO_PRT6, &port6_cfg);
	
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_STRONG_IN_OFF, 1, P6_6_GPIO);
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 7, CY_GPIO_DM_STRONG_IN_OFF, 1, P6_7_GPIO);
	Cy_SysLib_Delay(50);
}

/*******************************************************************************
*Function description:
* Generates the start condition of the I2C protocol.
*
*Input parameters:
* Not has input parameters.
*
*Return:
* Not return.
*
*Notes:
* It will be executed before any data transmission
********************************************************************************/
void I2C__P_StartCondition(void)
{
	/*Pull down de data signal*/
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_STRONG_IN_OFF, 0, P6_6_GPIO);

	/*Delay to give it time to settle*/
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);

	/*Pull down the clock signal*/
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 7, CY_GPIO_DM_STRONG_IN_OFF, 0, P6_7_GPIO);
}/* void I2C__P_StartCondition(void) */

/*******************************************************************************
*Function description:
* Generates the start condition of the I2C protocol in the read routine after
* relinquishing the SDA and SCL signals.
*
*Input parameters:
* Not has input parameters.
*
*Return:
* Not return.
*
*Notes:
* It will be executed after sending lMemAdd in read routine.
********************************************************************************/
void I2C__P_ReStartCondition(void)
{ 
	/* Pull up data line */
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_STRONG_IN_OFF, 1, P6_6_GPIO);

	/* Give it time to settle */
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);

	/* Pull up clock line */
	Cy_GPIO_Write(GPIO_PRT6, 7, 1);

	/* Give it time to settle */
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);

	/* Pull down data line */
	Cy_GPIO_Write(GPIO_PRT6, 6, 0);

	/* Give it time to settle */
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);

	/* Pull down clock line */
	Cy_GPIO_Write(GPIO_PRT6, 7, 0);

} /* void I2C__p_ReStartCondition(void) */


/*******************************************************************************
*Function description:
* Writes the data
*
*Input parameters:
* lAddressI2C: Address to write data to
* pStartBytes: Pointer to buffer
* lBytesNumber: Number of bytes to send
*
*Return:
* Returns a 1 if fail
*
*Notes:
*.
********************************************************************************/
uint8_t i2c_write(uint8_t lAddressI2C, uint8_t lBytesNumber, uint8_t * pStartBytes)
{ 
	/* Generates start condition to initiate communication */
	I2C__P_StartCondition();

	/* Sends I/O expander address, shifting 1 bit into position */
	I2C__P_SendByte(lAddressI2C<<1);

	/* Checks if ACK is received */
	if(!I2C__P_GetACK())
	{
	/* If not, stops communication and returns 0 for NACK */
	I2C__P_StopCondition();
	return(1);
	}

	/* Initiates reverse counting to send lBytesNumber bytes */
	while(lBytesNumber--)
	{
	/* Sends the pointer to TX_BUFFER_I2C3[5] */
	I2C__P_SendByte(*pStartBytes);

	/* Checks if ACK is received */
	if(!I2C__P_GetACK())
	{
	/* If not, stops communication and returns 0 for NACK */ 
	I2C__P_StopCondition();
	return(2);
	}
	/* Selects the next bit to be sent */
	pStartBytes++;
	}
	/* Stops communication after the writting is complete */
	I2C__P_StopCondition();

	return(0);


}


/*******************************************************************************
*Function description:
* Sends the bytes specified in the write function
*
*Input parameters:
* Byte: Variable in which the bytes are held to be sent in write function
*
*Return:
* Returns 1 to ACK
*
*Notes:
*.
********************************************************************************/
uint8_t I2C__P_SendByte(uint8_t Byte)
{
	/* Variable for counting the bits sent*/
	uint8_t count;
	
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_STRONG_IN_OFF, 0, P6_6_GPIO);
	/* Loop used for sending eight bits, one at a time */
	for(count = 0;count < 8; count++)
	{
	/* If the byte received is 0 holds the SDA signal down */
	if((Byte&0x80) == 0)
	{
		Cy_GPIO_Write(GPIO_PRT6, 6, 0);
	}
	/* Else release the line so data can be received */
	else
	{
		Cy_GPIO_Write(GPIO_PRT6, 6, 1);
	}
	/* Saves the data in the variable and shifts next bit into position */
	Byte = Byte <<1 ;
	/* Delay and clock pulse to ensure the data reception */
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);
	I2C__P_Clock();
	}
	/* Release data line for ACK */
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_PULLUP, 0, P6_6_GPIO);
	return(1);
} /* uint8_t I2C__P_SendByte(uint8_t Byte) */


 
/*******************************************************************************
*Function description:
* Checks the status of SDA LAT and sends ACK or NACK depending of that status
*
*Input parameters:
* Has not input parameters
*
*Return:
* Returns 1 to ACK
* Return 0 to NACK
*
*Notes:
*.
********************************************************************************/
uint8_t I2C__P_GetACK(void)
{
	/* Samples SDA LAT */
	if(Cy_GPIO_Read(GPIO_PRT6, 6) == 1)
	{
	/*If 1, return NACK*/
	return(0);
	}
	else
	{
	/* Else, pulses de clock and returns ACK */
	I2C__P_Clock();
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1); //-- Minimum Clock Low Time
	}

	return(1);
 
} /* uint8_t I2C__P_GetACK(void) */



/*******************************************************************************
*Function description:
* Generates the stop condition to end communication
*
*Input parameters:
* Has not input parameters
*
*Return:
* Has no return
*
*Notes:
*.
********************************************************************************/
void I2C__P_StopCondition(void)
{
	/* Pull down SDA line */
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_STRONG_IN_OFF, 0, P6_6_GPIO);
	/* Pull down SCL line */
	Cy_GPIO_Write(GPIO_PRT6, 7, 0);
	/* Give it time to settle */ 
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);
	/* Releases SCL line */
	Cy_GPIO_Write(GPIO_PRT6, 7, 1);
	/* Give it time to settle */
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);
	/* Releases SDA line */
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_STRONG_IN_OFF, 1, P6_6_GPIO);

} /* void I2C__P_StopRoutine(void) */



/*******************************************************************************
*Function description:
* Pulses SCL to generate a clock signal
*
*Input parameters:
* Has not input parameters
*
*Return:
* Has no return
*
*Notes:
*.
********************************************************************************/
void I2C__P_Clock(void)
{
	/* Delay to avoid signals to bed mixed*/
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);
	/* Pulls up SCL line */
	Cy_GPIO_Write(GPIO_PRT6, 7, 1);
	/* Give it time to settle */
	Cy_SysLib_DelayUs(DELAY_LONG);//__delay_us(2);
	/* Pulls down SCL line */
	Cy_GPIO_Write(GPIO_PRT6, 7, 0);
	/* Give it time to settle */
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);
 
} /* void I2C__P_Clock(void)*/



/*******************************************************************************
*Function description:
* Reads the data received
*
*Input parameters:
* lAddressI2C: Address to read data from
* pStartBytes: Pointer to buffer
* lBytesNumber: Number of bytes to read
*Return:
* Return 1 for fail
*
*Notes:
*.
********************************************************************************/
uint8_t i2c_read(uint8_t lAddressI2C, uint8_t lBytesNumber, uint8_t * pStartBytes)
{
	/* Variable used for counting the number of bits sent */
	uint8_t lIndex;

	/* Generates start condition */
	I2C__P_StartCondition();

	/* Sends expander address, shifting 1 bit into position */
	I2C__P_SendByte(lAddressI2C << 1);

	/* Checks if ACK is received */
	if(I2C__P_GetACK()==0)
	{
	/* If not, stops communication and return 0 for NACK */
	I2C__P_StopCondition();
	return(1); 
	}

	/* Restarts I2C to begin the reading */
	I2C__P_ReStartCondition();

	/* Sends expander addres, shifting 1 bit into position and does an OR with 1 bit to enable reading */
	I2C__P_SendByte((lAddressI2C << 1) | (0x01));

	/* Checks if ACK is received */
	if(I2C__P_GetACK()==0)
	{
	/* If not, stops communication and return 0 for NACK */
	I2C__P_StopCondition();
	return(1);
	}

	/* Generates a reverse counting loop to send lBytesNumber bytes*/ 
	for(lIndex=0;lIndex<lBytesNumber;lIndex++ )
	{
	/* Equals the RX_BUFFER_I2C2[5] to the bytes received in I2C__P_GetByte(); function */
	pStartBytes[lIndex]=I2C__P_GetByte();

	/* While the lIndex countig is below lBytesNumber - 1 sends ACK for each byte received */
	if(lIndex<(lBytesNumber-1))
	{
	/* Forces and ACK if the condition is fulfilled */
	I2C__P_SendACK();
	}
	/* Else sends NACK */ 
	else
	{
	/* Forces NACK if else */ 
	I2C__P_NACK();
	}
	}

	/* Stops communitacion after the reading is completed */ 
	I2C__P_StopCondition();

	return(0);

}



/*******************************************************************************
*Function description:
* Gets the bytes to be read from the SDA port
*
*Input parameters:
* Has not input parameters
*
*Return:
* Byte: variable used to store the read bytes in
*
*Notes:
*.
********************************************************************************/
uint8_t I2C__P_GetByte(void)
{
	/* Variable used for counting the number of bytes read */
	int8_t count;

	/* Variable used to store the read bytes in */
	int8_t Byte = 0;

	/* Releases SDA line to allow the bytes to be read */
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_PULLUP, 0, P6_6_GPIO);
	//Cy_GPIO_SetDrivemode(GPIO_PRT6, 6, CY_GPIO_DM_HIGHZ);

	/* Initiares a loop to read 8 bits */
	for(count=0;count<8;count++)
	{
	/* Equals the Byte variable to Byte OR data held in SDA port. ****fs the the next bits into position as the counting goes on */
	Byte = Byte | ( Cy_GPIO_Read(GPIO_PRT6, 6) << (7-count) ); //-- Shift next bit into position

	/* Delay and clock pulse to ensure the data reception */
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);
	I2C__P_Clock(); //-- Pulse the clock 
	}
	/* Returns the data held in Byte variable to be used in I2C__P_ReadIOExpander function */
	return(Byte);

} /* I2C__P_GetByte */



/*******************************************************************************
*Function description:
* Forces an ACK to read incoming data
*
*Input parameters:
* Has not input parameters
*
*Return:
* Has no return
*
*Notes:
*.
********************************************************************************/
void I2C__P_SendACK(void)
{ 
	/* Pulls down SDA, generating ACK */
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_STRONG_IN_OFF, 0, P6_6_GPIO);

	/* Give it time to settle */
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);

	/* Pulse the clock */
	I2C__P_Clock();

	/* Release SDA */
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_STRONG_IN_OFF, 1, P6_6_GPIO);
	//Cy_GPIO_SetDrivemode(GPIO_PRT6, 6, CY_GPIO_DM_HIGHZ);

	/* Give it time to settle */
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);

} /* void I2C__P_SendACK(void) */ 



/*******************************************************************************
*Function description:
* Forces a NACK to end communication
*
*Input parameters:
* Has not input parameters
*
*Return:
* Has no return
*
*Notes:
*.
********************************************************************************/
void I2C__P_NACK(void)
{ 
	/* Release SDA*/
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_STRONG_IN_OFF, 1, P6_6_GPIO);

	/* Give it time to settle */ 
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);

	/* Pulse the clock */
	I2C__P_Clock();

	/* Pulls down SDA, genrating ACK */
	Cy_GPIO_Pin_FastInit(GPIO_PRT6, 6, CY_GPIO_DM_STRONG_IN_OFF, 0, P6_6_GPIO);

	/* Give it time to settle */
	Cy_SysLib_DelayUs(DELAY_SHORT);//__delay_us(1);

} /* void I2C__P_NACK(void) */
